mod error;
mod game;
mod launcher;
mod utils;

use crate::game::download_game;
use crate::launcher::download_launcher;
use indicatif::MultiProgress;
use indicatif_log_bridge::LogWrapper;
use log::info;

fn main() {
    let logger = colog::default_builder()
        .filter_level(log::LevelFilter::Info)
        .build();
    let multi = MultiProgress::new();
    LogWrapper::new(multi.clone(), logger).try_init().unwrap();

    wicked_waifus_downloader_lib::enable_progress_bar(multi);

    test_launcher_download();
    test_game_download(false, false, true);
}

fn test_launcher_download() {
    let launcher_config = wicked_waifus_downloader_lib::request_launcher(
        "https://prod-cn-alicdn-gamestarter.kurogame.com/launcher/launcher/10008_Pa0Q0EMFxukjEqX33pF9Uyvdc8MaGPSz/G152/index.json",
        None,
    ).unwrap();
    info!("{:#?}", launcher_config);

    download_launcher(launcher_config, None, "test_launcher_dl").unwrap();
}

fn test_game_download(use_predownload: bool, use_chunked: bool, full: bool) {
    let game_config = wicked_waifus_downloader_lib::request_game(
        "https://prod-cn-alicdn-gamestarter.kurogame.com/launcher/game/G152/10008_Pa0Q0EMFxukjEqX33pF9Uyvdc8MaGPSz/index.json",
        None,
    ).unwrap();
    info!("{:#?}", game_config);

    download_game(
        use_predownload,
        use_chunked,
        full,
        game_config,
        None,
        "test_game_dl",
    )
    .unwrap();
}
