use crate::error::Error;
use constant_time_eq::constant_time_eq;
use md5::{Digest, Md5};
use std::io::Write;
use wicked_waifus_downloader_domain::CdnInfo;

pub(crate) fn save_file_safe(prefix: &str, filename: &str, data: &[u8]) -> Result<String, Error> {
    let root = std::path::Path::new(prefix);
    let location = root.join(filename);
    let parent = location.parent().unwrap();
    std::fs::create_dir_all(parent)?;

    let mut f = std::fs::OpenOptions::new()
        .create(true)
        .write(true)
        .truncate(true)
        .open(&location)?;
    f.write_all(data)?;
    f.flush()?;

    Ok(location.to_string_lossy().to_string())
}

pub(crate) fn sort_cdn_list(cdn_list: &mut Vec<CdnInfo>) {
    // Highest priority first
    // TODO: Implement K1 and K2 sorting
    cdn_list.sort_by(|a, b| b.p.cmp(&a.p));
}

pub(crate) fn verify_download(
    data: &[u8],
    expected_length: u64,
    expected_md5: &[u8],
) -> Result<(), Error> {
    let actual_length = data.len() as u64;
    if actual_length != expected_length {
        return Err(Error::LengthMismatch(expected_length, actual_length));
    }
    let actual_md5 = Md5::digest(data);
    let actual_md5_bytes = actual_md5.as_slice();
    match constant_time_eq(expected_md5, actual_md5_bytes) {
        true => Ok(()),
        false => Err(Error::CrcMismatch(
            hex::encode(expected_md5),
            hex::encode(actual_md5_bytes),
        )),
    }
}
