use crate::{CdnInfo, Experiment, LanguageMap};
use serde::{Deserialize, Serialize};

#[derive(Debug, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct GameConfig {
    pub chunk_download_switch: Option<u32>,
    pub default: GameDefaultConfig,
    pub key_file_check_switch: Option<u32>,
    pub predownload_switch: Option<u32>,
    pub predownload: Option<ResourcesData>, // TODO: Might need changes
    #[serde(rename = "RHIOptionSwitch")]
    pub rhioption_switch: Option<u32>,
    #[serde(rename = "RHIOptionList")]
    pub rhioption_list: Option<Vec<RHIOption>>,
    pub resources_login: Option<ResourcesLogin>,
    pub experiment: Option<Experiment>,
    pub check_exe_is_running: Option<u32>,
    pub hash_cache_check_acc_switch: u32,
    pub key_file_check_list: Option<Vec<String>>,
    pub fingerprints: Option<Vec<String>>,
}

/// The order V2 before V1, is because they share a common structure and only V2 has extra fields.
/// If V1 it's first will be included in V2 too
#[derive(Serialize, Deserialize, Debug)]
#[serde(untagged)]
pub enum GameDefaultConfig {
    GameDefaultConfigV2(GameDefaultConfigV2),
    GameDefaultConfigV1(GameDefaultConfigV1),
}

#[derive(Debug, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct GameDefaultConfigV1 {
    pub cdn_list: Vec<CdnInfo>,
    #[serde(flatten)]
    pub resources_data: ResourcesData,
}

#[derive(Debug, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct GameDefaultConfigV2 {
    pub cdn_list: Vec<CdnInfo>,
    #[serde(flatten)]
    pub resources_data: ResourcesData,
    pub config: GameDefaultConfigV2Config,
}

#[derive(Debug, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct ResourcesData {
    pub changelog: LanguageMap,
    pub resources: String,
    pub resources_base_path: String,
    pub resources_diff: ResourcesDiff,
    pub resources_exclude_path: Vec<String>,
    pub resources_exclude_path_need_update: Vec<String>,
    pub sample_hash_switch: u32,
    pub version: String,
}

#[derive(Debug, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct GameDefaultConfigV2Config {
    pub index_file_md5: String,
    pub un_compress_size: u64,
    pub base_url: String,
    pub size: u64,
    pub patch_type: String, // TODO: Enum
    pub index_file: String,
    pub version: String,
    pub patch_config: Vec<GameDefaultConfigV2PatchConfig>,
}

#[derive(Debug, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct GameDefaultConfigV2PatchConfig {
    pub index_file_md5: String,
    pub un_compress_size: u64,
    pub ext: GameDefaultConfigV2PatchConfigExt,
    pub base_url: String,
    pub size: u64,
    pub index_file: String,
    pub version: String,
}

#[derive(Debug, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct GameDefaultConfigV2PatchConfigExt {
    pub max_file_size: u64,
}

#[derive(Debug, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct ResourcesDiff {
    pub current_game_info: GameInfo,
    pub previous_game_info: Option<GameInfo>,
}

#[derive(Debug, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct GameInfo {
    pub file_name: String,
    pub md5: String,
    pub version: String,
}

#[derive(Debug, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct RHIOption {
    pub cmd_option: String,
    pub is_show: u32,
    pub text: LanguageMap,
}

#[derive(Debug, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct ResourcesLogin {
    pub host: String,
    pub login_switch: u32,
}
